@@ -1,8 +1,8 @@
 <?php
 
 /*
- * Copyright (c) 2015-2018 Franco Fichtner <franco@opnsense.org>
- * Copyright (c) 2015-2016 Deciso B.V.
+ * Copyright (c) 2015-2019 Franco Fichtner <franco@opnsense.org>
+ * Copyright (c) 2015-2018 Deciso B.V.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,15 +46,72 @@ class FirmwareController extends ApiControllerBase
      */
     protected function formatBytes($bytes)
     {
-        if ($bytes >= (1024 * 1024 * 1024)) {
-            return sprintf("%d GB", $bytes / (1024 * 1024 * 1024));
-        } elseif ($bytes >= 1024 * 1024) {
-            return sprintf("%d MB", $bytes / (1024 * 1024));
-        } elseif ($bytes >= 1024) {
-            return sprintf("%d KB", $bytes / 1024);
-        } else {
-            return sprintf("%d bytes", $bytes);
+        if (preg_match('/[^0-9]/', $bytes)) {
+            /* already processed */
+            return $bytes;
         }
+        if ($bytes >= (1024 * 1024 * 1024)) {
+            return sprintf('%.1F%s', $bytes / (1024 * 1024 * 1024), 'GiB');
+        } elseif ($bytes >= 1024 * 1024) {
+            return sprintf('%.1F%s', $bytes / (1024 * 1024), 'MiB');
+        } elseif ($bytes >= 1024) {
+            return sprintf('%.1F%s', $bytes / 1024, 'KiB');
+        } else {
+            return sprintf('%d%s', $bytes, 'B');
+        }
+    }
+
+    /**
+     * process plugins for config-bound cache (name only)
+     * @param string $name to process
+     * @param string $action taken
+     */
+    protected function processPlugin($name, $action)
+    {
+        if (strpos($name, 'os-') !== 0) {
+            /* not a plugin, don't care */
+            return;
+        }
+
+        $config = Config::getInstance()->object();
+
+        if (!isset($config->system->firmware)) {
+            $config->system->addChild('firmware');
+        }
+
+        $plugins = array();
+        if (!isset($config->system->firmware->plugins)) {
+            $config->system->firmware->addChild('plugins');
+        } else {
+            $plugins = explode(',', (string)$config->system->firmware->plugins);
+        }
+        $plugins = array_flip($plugins);
+
+        switch ($action) {
+            case 'install':
+            case 'reinstall':
+                $plugins[$name] = 'hello';
+                break;
+            case 'remove':
+                if (isset($plugins[$name])) {
+                    unset($plugins[$name]);
+                }
+                break;
+            default:
+                break;
+        }
+
+        $config->system->firmware->plugins = implode(',', array_keys($plugins));
+
+        if (empty($config->system->firmware->plugins)) {
+            unset($config->system->firmware->plugins);
+        }
+
+        if (!@count($config->system->firmware->children())) {
+            unset($config->system->firmware);
+        }
+
+        Config::getInstance()->save();
     }
 
     /**
@@ -74,23 +131,13 @@ class FirmwareController extends ApiControllerBase
         $backend = new Backend();
         $type_have = trim($backend->configdRun('firmware type name'));
         $backend->configdRun('firmware changelog fetch');
+        $args = array();
 
         if (!empty($type_have) && $type_have !== $type_want) {
-            $type_ver = trim($backend->configdRun('firmware type version ' . escapeshellarg($type_want)));
-            return array(
-                'status_msg' => gettext('The release type requires an update.'),
-                'all_packages' => array($type_want => array(
-                    'new' => empty($type_ver) ? gettext('N/A') : $type_ver,
-                    'reason' => gettext('new'),
-                    'old' => gettext('N/A'),
-                    'name' => $type_want,
-                )),
-                'status_upgrade_action' => 'rel',
-                'status' => 'ok',
-            );
+            $args[] = $type_want;
         }
 
-        $response = json_decode(trim($backend->configdRun('firmware check')), true);
+        $response = json_decode(trim($backend->configdpRun('firmware check', $args)), true);
 
         if ($response != null) {
             $packages_size = !empty($response['download_size']) ? $response['download_size'] : 0;
@@ -126,10 +173,97 @@ class FirmwareController extends ApiControllerBase
 
             $download_size = $this->formatBytes($packages_size + $sets_size);
 
-            if (array_key_exists('connection', $response) && $response['connection'] == 'error') {
-                $response['status_msg'] = gettext('Connection error.');
+            $sorted = array();
+
+            foreach (array('new_packages', 'reinstall_packages', 'upgrade_packages',
+                'downgrade_packages', 'remove_packages') as $pkg_type) {
+                if (isset($response[$pkg_type])) {
+                    foreach ($response[$pkg_type] as $value) {
+                        switch ($pkg_type) {
+                            case 'downgrade_packages':
+                                $sorted[$value['name']] = array(
+                                    'reason' => gettext('downgrade'),
+                                    'old' => $value['current_version'],
+                                    'new' => $value['new_version'],
+                                    'name' => $value['name'],
+                                );
+                                break;
+                            case 'new_packages':
+                                $sorted[$value['name']] = array(
+                                    'new' => $value['version'],
+                                    'reason' => gettext('new'),
+                                    'name' => $value['name'],
+                                    'old' => gettext('N/A'),
+                                );
+                                break;
+                            case 'reinstall_packages':
+                                $sorted[$value['name']] = array(
+                                    'reason' => gettext('reinstall'),
+                                    'new' => $value['version'],
+                                    'old' => $value['version'],
+                                    'name' => $value['name'],
+                                );
+                                break;
+                            case 'remove_packages':
+                                $sorted[$value['name']] = array(
+                                    'reason' => gettext('obsolete'),
+                                    'new' => gettext('N/A'),
+                                    'old' => $value['version'],
+                                    'name' => $value['name'],
+                                );
+                                break;
+                            case 'upgrade_packages':
+                                $sorted[$value['name']] = array(
+                                    'reason' => gettext('upgrade'),
+                                    'old' => empty($value['current_version']) ?
+                                        gettext('N/A') : $value['current_version'],
+                                    'new' => $value['new_version'],
+                                    'name' => $value['name'],
+                                );
+                                break;
+                            default:
+                                /* undefined */
+                                break;
+                        }
+                    }
+                }
+            }
+
+            uksort($sorted, function ($a, $b) {
+                return strnatcasecmp($a, $b);
+            });
+
+            $response['all_packages'] = $sorted;
+
+            if (count($args)) {
+                $response['status_msg'] = gettext('The release type requires an update.');
+                $response['status_upgrade_action'] = 'rel';
+                $response['status'] = 'ok';
+            } elseif (array_key_exists('connection', $response) && $response['connection'] == 'busy') {
+                $response['status_msg'] = gettext('The package manager is not responding.');
                 $response['status'] = 'error';
-            } elseif (array_key_exists('repository', $response) && $response['repository'] == 'error') {
+            } elseif (array_key_exists('connection', $response) && $response['connection'] == 'unresolved') {
+                $response['status_msg'] = gettext('No address record found for the selected mirror.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('connection', $response) && $response['connection'] == 'unauthenticated') {
+                $response['status_msg'] = gettext('Could not authenticate the selected mirror.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('connection', $response) && $response['connection'] == 'misconfigured') {
+                $response['status_msg'] = gettext('The current package configuration is invalid.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('connection', $response) && $response['connection'] == 'timeout') {
+                $response['status_msg'] = gettext('Timeout while connecting to the selected mirror.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('connection', $response) && $response['connection'] != 'ok') {
+                $response['status_msg'] = gettext('An error occurred while connecting to the selected mirror.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('repository', $response) && $response['repository'] == 'untrusted') {
+                $response['status_msg'] = gettext('Could not verify the repository fingerprint.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('repository', $response) && $response['repository'] == 'revoked') {
+                $response['status_msg'] = gettext('The repository fingerprint has been revoked.');
+                $response['status'] = 'error';
+            } elseif (array_key_exists('repository', $response) && $response['repository'] != 'ok') {
                 $response['status_msg'] = gettext('Could not find the repository on the selected mirror.');
                 $response['status'] = 'error';
             } elseif (array_key_exists('updates', $response) && $response['updates'] == 0) {
@@ -164,73 +298,14 @@ class FirmwareController extends ApiControllerBase
                         gettext('This update requires a reboot.')
                     );
                 }
+            } else {
+                $response['status_msg'] = gettext('Unknown firmware status encountered.');
+                $response['status'] = 'unknown';
             }
-
-            $sorted = array();
-
-            /*
-             * new_packages: array with { name: <package_name>, version: <package_version> }
-             * reinstall_packages: array with { name: <package_name>, version: <package_version> }
-             * upgrade_packages: array with { name: <package_name>,
-             *     current_version: <current_version>, new_version: <new_version> }
-             * downgrade_packages: array with { name: <package_name>,
-             *     current_version: <current_version>, new_version: <new_version> }
-             */
-            foreach (array('new_packages', 'reinstall_packages', 'upgrade_packages',
-                'downgrade_packages') as $pkg_type) {
-                if (isset($response[$pkg_type])) {
-                    foreach ($response[$pkg_type] as $value) {
-                        switch ($pkg_type) {
-                            case 'downgrade_packages':
-                                $sorted[$value['name']] = array(
-                                    'reason' => gettext('downgrade'),
-                                    'old' => $value['current_version'],
-                                    'new' => $value['new_version'],
-                                    'name' => $value['name'],
-                                );
-                                break;
-                            case 'new_packages':
-                                $sorted[$value['name']] = array(
-                                    'new' => $value['version'],
-                                    'reason' => gettext('new'),
-                                    'name' => $value['name'],
-                                    'old' => gettext('N/A'),
-                                );
-                                break;
-                            case 'reinstall_packages':
-                                $sorted[$value['name']] = array(
-                                    'reason' => gettext('reinstall'),
-                                    'new' => $value['version'],
-                                    'old' => $value['version'],
-                                    'name' => $value['name'],
-                                );
-                                break;
-                            case 'upgrade_packages':
-                                $sorted[$value['name']] = array(
-                                    'reason' => gettext('upgrade'),
-                                    'old' => empty($value['current_version']) ?
-                                        gettext('N/A') : $value['current_version'],
-                                    'new' => $value['new_version'],
-                                    'name' => $value['name'],
-                                );
-                                break;
-                            default:
-                                /* undefined */
-                                break;
-                        }
-                    }
-                }
-            }
-
-            uksort($sorted, function ($a, $b) {
-                return strnatcasecmp($a, $b);
-            });
-
-            $response['all_packages'] = $sorted;
         } else {
             $response = array(
                 'status_msg' => gettext('Firmware status check was aborted internally. Please try again.'),
-                'status' => 'unknown'
+                'status' => 'unknown',
             );
         }
 
@@ -439,6 +514,7 @@ class FirmwareController extends ApiControllerBase
             $pkg_name = $filter->sanitize($pkg_name, "pkgname");
             // execute action
             $response['msg_uuid'] = trim($backend->configdpRun("firmware reinstall", array($pkg_name), true));
+            $this->processPlugin($pkg_name, 'reinstall');
         } else {
             $response['status'] = 'failure';
         }
@@ -468,6 +544,7 @@ class FirmwareController extends ApiControllerBase
             $pkg_name = $filter->sanitize($pkg_name, "pkgname");
             // execute action
             $response['msg_uuid'] = trim($backend->configdpRun("firmware install", array($pkg_name), true));
+            $this->processPlugin($pkg_name, 'install');
         } else {
             $response['status'] = 'failure';
         }
@@ -497,6 +574,7 @@ class FirmwareController extends ApiControllerBase
             $pkg_name = $filter->sanitize($pkg_name, "pkgname");
             // execute action
             $response['msg_uuid'] = trim($backend->configdpRun("firmware remove", array($pkg_name), true));
+            $this->processPlugin($pkg_name, 'remove');
         } else {
             $response['status'] = 'failure';
         }
@@ -609,6 +687,7 @@ class FirmwareController extends ApiControllerBase
     /**
      * query package details
      * @return array
+     * @throws \Exception
      */
     public function detailsAction($package)
     {
@@ -640,14 +719,16 @@ class FirmwareController extends ApiControllerBase
     {
         $this->sessionClose(); // long running action, close session
 
-        $keys = array('name', 'version', 'comment', 'flatsize', 'locked', 'license');
+        $keys = array('name', 'version', 'comment', 'flatsize', 'locked', 'license', 'repository', 'origin');
         $backend = new Backend();
         $response = array();
 
-        /* allows us to select UI features based on product state */
-        $response['product_version'] = trim(file_get_contents('/usr/local/opnsense/version/opnsense'));
-        $response['product_name'] = trim(file_get_contents('/usr/local/opnsense/version/opnsense.name'));
+        $version = explode(' ', trim(shell_exec('opnsense-version -nv')));
+        foreach (array('product_name' => 0, 'product_version' => 1) as $result => $index) {
+            $response[$result] = !empty($version[$index]) ? $version[$index] : 'unknown';
+        }
 
+        /* allows us to select UI features based on product state */
         $devel = explode('-', $response['product_name']);
         $devel = count($devel) == 2 ? $devel[1] == 'devel' : false;
 
@@ -671,16 +752,19 @@ class FirmwareController extends ApiControllerBase
                     $translated[$key] = $expanded[$index++];
                     if (empty($translated[$key])) {
                         $translated[$key] = gettext('N/A');
+                    } elseif ($key == 'flatsize') {
+                        $translated[$key] = $this->formatBytes($translated[$key]);
                     }
                 }
 
                 /* mark remote packages as "provided", local as "installed" */
-                $translated['provided'] = $type == 'remote' ? "1" : "0";
-                $translated['installed'] = $type == 'local' ? "1" : "0";
+                $translated['provided'] = $type == 'remote' ? '1' : '0';
+                $translated['installed'] = $type == 'local' ? '1' : '0';
                 if (isset($packages[$translated['name']])) {
                     /* local iteration, mark package provided */
-                    $translated['provided'] = "1";
+                    $translated['provided'] = '1';
                 }
+                $translated['path'] = "{$translated['repository']}/{$translated['origin']}";
                 $packages[$translated['name']] = $translated;
 
                 /* figure out local and remote plugins */
@@ -722,7 +806,16 @@ class FirmwareController extends ApiControllerBase
         if ($changelogs == null) {
             $changelogs = array();
         } else {
-            foreach ($changelogs as &$changelog) {
+            /* development strategy for changelog slightly differs from above */
+            $devel = preg_match('/^\d+\.\d+\.[a-z]/i', $response['product_version']) ? true : false;
+
+            foreach ($changelogs as $index => &$changelog) {
+                /* skip development items */
+                if (!$devel && preg_match('/^\d+\.\d+\.[a-z]/i', $changelog['version'])) {
+                    unset($changelogs[$index]);
+                    continue;
+                }
+
                 /* rewrite dates as ISO */
                 $date = date_parse($changelog['date']);
                 $changelog['date'] = sprintf('%04d-%02d-%02d', $date['year'], $date['month'], $date['day']);
@@ -745,49 +838,80 @@ class FirmwareController extends ApiControllerBase
     public function getFirmwareOptionsAction()
     {
         $this->sessionClose(); // long running action, close session
-
-        /* XXX we might want to move these into configuration files later */
         $mirrors = array();
-        $mirrors[''] = '(default)';
-        $mirrors['https://opnsense.aivian.org'] = 'Aivian (Shaoxing, CN)';
-        $mirrors['https://opnsense-update.deciso.com'] = 'Deciso (NL, Commercial)';
-        $mirrors['https://mirror.dns-root.de/opnsense'] = 'dns-root.de (Cloudflare CDN)';
-        $mirrors['https://opnsense.c0urier.net'] = 'c0urier.net (Lund, SE)';
-        $mirrors['https://ftp.yzu.edu.tw/opnsense'] = 'Dept. of CSE, Yuan Ze University (Taoyuan City, TW)';
-        $mirrors['https://fourdots.com/mirror/OPNSense'] = 'FourDots (Belgrade, RS)';
-        $mirrors['https://opnsense-mirror.hiho.ch'] = 'HiHo (Zurich, CH)';
-        $mirrors['https://opnsense.ieji.de'] = 'ieji.de (Frankfurt, DE)';
-        $mirrors['http://mirror.ams1.nl.leaseweb.net/opnsense'] = 'LeaseWeb (Amsterdam, NL)';
-        $mirrors['http://mirror.fra10.de.leaseweb.net/opnsense'] = 'LeaseWeb (Frankfurt, DE)';
-        $mirrors['http://mirror.sfo12.us.leaseweb.net/opnsense'] = 'LeaseWeb (San Francisco, US)';
-        $mirrors['http://mirror.wdc1.us.leaseweb.net/opnsense'] = 'LeaseWeb (Washington, D.C., US)';
-        $mirrors['http://mirrors.nycbug.org/pub/opnsense'] = 'NYC*BUG (New York, US)';
-        $mirrors['http://pkg.opnsense.org'] = 'OPNsense (Amsterdam, NL)';
-        $mirrors['http://mirror.ragenetwork.de/opnsense'] = 'RageNetwork (Munich, DE)';
-        $mirrors['http://mirror.upb.edu.co/opnsense'] = 'Universidad Pontificia Bolivariana (Medellin, CO)';
-        $mirrors['http://mirror.venturasystems.tech/opnsense'] = 'Ventura Systems (Medellin, CO)';
-        $mirrors['http://mirror.wjcomms.co.uk/opnsense'] = 'WJComms (London, GB)';
-
-        $has_subscription = array();
-        $has_subscription[] = 'https://opnsense-update.deciso.com';
-
         $flavours = array();
-        $flavours[''] = '(default)';
-        $flavours['libressl'] = 'LibreSSL';
-        $flavours['latest'] = 'OpenSSL';
-
         $families = array();
-        $families[''] = gettext('Production');
-        $families['devel'] = gettext('Development');
-
+        $has_subscription = array();
+        $allow_custom = false;
+        // parse firmware options
+        foreach (glob(__DIR__. "/repositories/*.xml") as $xml) {
+            $repositoryXml = simplexml_load_file($xml);
+            if ($repositoryXml === false || $repositoryXml->getName() != 'firmware') {
+                syslog(LOG_ERR, 'unable to parse firmware file '. $xml);
+            } else {
+                if (isset($repositoryXml->mirrors->mirror)) {
+                    if (isset($repositoryXml->mirrors->attributes()->allow_custom)) {
+                        $allow_custom = (strtolower($repositoryXml->mirrors->attributes()->allow_custom) == "true");
+                    }
+                    foreach ($repositoryXml->mirrors->mirror as $mirror) {
+                        $mirrors[(string)$mirror->url] = (string)$mirror->description;
+                        $attr = $mirror->attributes();
+                        if (isset($attr->has_subscription) && strtolower($attr->has_subscription) == "true") {
+                            $has_subscription[] = (string)$mirror->url;
+                        }
+                    }
+                }
+                if (isset($repositoryXml->flavours->flavour)) {
+                    foreach ($repositoryXml->flavours->flavour as $flavour) {
+                        $flavours[(string)$flavour->name] = (string)$flavour->description;
+                    }
+                }
+                if (isset($repositoryXml->families->family)) {
+                    foreach ($repositoryXml->families->family as $family) {
+                        $families[(string)$family->name] = (string)$family->description;
+                    }
+                }
+            }
+        }
         return array(
             'has_subscription' => $has_subscription,
             'flavours' => $flavours,
             'families' => $families,
             'mirrors' => $mirrors,
+            'allow_custom' => $allow_custom
         );
     }
 
+    /**
+     * Validate firmware options
+     * @param $selectedMirror selected mirror url
+     * @param $selectedFlavour selected flavour
+     * @param $selectedType selected family type
+     * @param $selSubscription selected subscription id
+     * @return array|bool is valid config
+     */
+    private function validateFirmwareOptions($selectedMirror, $selectedFlavour, $selectedType, $selSubscription)
+    {
+        $validOptions = $this->getFirmwareOptionsAction();
+        if ($validOptions['allow_custom']) {
+            // custom options allowed, skip validations
+            return true;
+        } elseif (!isset($validOptions['flavours'][$selectedFlavour])) {
+            syslog(LOG_ERR, 'unable to set invalid firmware flavour '. $selectedFlavour);
+            return false;
+        } elseif (!isset($validOptions['families'][$selectedType])) {
+            syslog(LOG_ERR, 'unable to set invalid firmware release type '. $selectedFlavour);
+            return false;
+        } elseif (!isset($validOptions['mirrors'][$selectedMirror])) {
+            syslog(LOG_ERR, 'unable to set invalid firmware mirror '. $selectedMirror);
+            return false;
+        } elseif (!empty($selSubscription) && !in_array($selectedMirror, $validOptions['has_subscription'])) {
+            syslog(LOG_ERR, 'unable to set subscription for firmware mirror '. $selectedMirror);
+            return false;
+        }
+        return true;
+    }
+
     /**
      * retrieve current firmware configuration options
      * @return array
@@ -834,48 +958,52 @@ class FirmwareController extends ApiControllerBase
             $selectedType = filter_var($this->request->getPost("type", null, ""), FILTER_SANITIZE_URL);
             $selSubscription = filter_var($this->request->getPost("subscription", null, ""), FILTER_SANITIZE_URL);
 
-            // config data without model, prepare xml structure and write data
-            if (!isset($config->system->firmware)) {
-                $config->system->addChild('firmware');
-            }
+            if ($this->validateFirmwareOptions($selectedMirror, $selectedFlavour, $selectedType, $selSubscription)) {
+                // config data without model, prepare xml structure and write data
+                if (!isset($config->system->firmware)) {
+                    $config->system->addChild('firmware');
+                }
 
-            if (!isset($config->system->firmware->mirror)) {
-                $config->system->firmware->addChild('mirror');
-            }
-            if (empty($selSubscription)) {
-                $config->system->firmware->mirror = $selectedMirror;
+                if (!isset($config->system->firmware->mirror)) {
+                    $config->system->firmware->addChild('mirror');
+                }
+                if (empty($selSubscription)) {
+                    $config->system->firmware->mirror = $selectedMirror;
+                } else {
+                    // prepend subscription
+                    $config->system->firmware->mirror = $selectedMirror . '/' . $selSubscription;
+                }
+                if (empty($config->system->firmware->mirror)) {
+                    unset($config->system->firmware->mirror);
+                }
+
+                if (!isset($config->system->firmware->flavour)) {
+                    $config->system->firmware->addChild('flavour');
+                }
+                $config->system->firmware->flavour = $selectedFlavour;
+                if (empty($config->system->firmware->flavour)) {
+                    unset($config->system->firmware->flavour);
+                }
+
+                if (!isset($config->system->firmware->type)) {
+                    $config->system->firmware->addChild('type');
+                }
+                $config->system->firmware->type = $selectedType;
+                if (empty($config->system->firmware->type)) {
+                    unset($config->system->firmware->type);
+                }
+
+                if (!@count($config->system->firmware->children())) {
+                    unset($config->system->firmware);
+                }
+
+                Config::getInstance()->save();
+
+                $backend = new Backend();
+                $backend->configdRun("firmware configure");
             } else {
-                // prepend subscription
-                $config->system->firmware->mirror = $selectedMirror . '/' . $selSubscription;
+                $response['status'] = "failed";
             }
-            if (empty($config->system->firmware->mirror)) {
-                unset($config->system->firmware->mirror);
-            }
-
-            if (!isset($config->system->firmware->flavour)) {
-                $config->system->firmware->addChild('flavour');
-            }
-            $config->system->firmware->flavour = $selectedFlavour;
-            if (empty($config->system->firmware->flavour)) {
-                unset($config->system->firmware->flavour);
-            }
-
-            if (!isset($config->system->firmware->type)) {
-                $config->system->firmware->addChild('type');
-            }
-            $config->system->firmware->type = $selectedType;
-            if (empty($config->system->firmware->type)) {
-                unset($config->system->firmware->type);
-            }
-
-            if (!@count($config->system->firmware->children())) {
-                unset($config->system->firmware);
-            }
-
-            Config::getInstance()->save();
-
-            $backend = new Backend();
-            $backend->configdRun("firmware configure");
         }
 
         return $response;
