@@ -1,32 +1,31 @@
 <?php
 
-/**
- *    Copyright (C) 2017 Deciso B.V.
+/*
+ * Copyright (C) 2017 Deciso B.V.
+ * All rights reserved.
  *
- *    All rights reserved.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *    Redistribution and use in source and binary forms, with or without
- *    modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
  *
- *    1. Redistributions of source code must retain the above copyright notice,
- *       this list of conditions and the following disclaimer.
- *
- *    2. Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
- *
- *    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- *    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- *    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- *    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
- *    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- *    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- *    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- *    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- *    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- *    POSSIBILITY OF SUCH DAMAGE.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
+
 namespace OPNsense\Firewall;
 
 /**
@@ -37,6 +36,7 @@ abstract class Rule
 {
     protected $rule = array();
     protected $interfaceMapping = array();
+    protected $ruleDebugInfo = array();
 
     /**
      * init Rule
@@ -49,6 +49,15 @@ abstract class Rule
         $this->rule = $conf;
     }
 
+    /**
+     * send text to debug log
+     * @param string debug log info
+     */
+    protected function log($line)
+    {
+        $this->ruleDebugInfo[] = $line;
+    }
+
     /**
      * output parsing
      * @param string $value field value
@@ -71,8 +80,8 @@ abstract class Rule
 
     /**
      * parse static text
-     * @param string $value field value, ignored
      * @param string $value static text
+     * @param string $text
      * @return string
      */
     protected function parseStaticText($value, $text)
@@ -83,6 +92,8 @@ abstract class Rule
     /**
      * parse boolean, return text from $valueTrue / $valueFalse
      * @param string $value field value
+     * @param $valueTrue
+     * @param string $valueFalse
      * @return string
      */
     protected function parseBool($value, $valueTrue, $valueFalse = "")
@@ -180,6 +191,7 @@ abstract class Rule
                 $this->convertAddress($rule);
                 // disable rule when interface not found
                 if (!empty($interface) && empty($this->interfaceMapping[$interface]['if'])) {
+                    $this->log("Interface {$interface} not found");
                     $rule['disabled'] = true;
                 }
                 yield $rule;
@@ -210,7 +222,12 @@ abstract class Rule
             $ruleTxt .= !empty($cmdout) && !empty($ruleTxt) ? " "  : "";
             $ruleTxt .= $cmdout;
         }
-        return $ruleTxt;
+        if (!empty($this->ruleDebugInfo)) {
+            $debugTxt = "#debug:". implode("|", $this->ruleDebugInfo) . "\n";
+        } else {
+            $debugTxt = "";
+        }
+        return $debugTxt . $ruleTxt;
     }
 
     /**
@@ -266,6 +283,7 @@ abstract class Rule
                         if (!Util::isAlias($port, true)) {
                             // unable to map port
                             $rule['disabled'] = true;
+                            $this->log("Unable to map port {$port}, empty?");
                         }
                     }
                 }
@@ -273,6 +291,7 @@ abstract class Rule
                     // couldn't convert address, disable rule
                     // dump all tag contents in target (from/to) for reference
                     $rule['disabled'] = true;
+                    $this->log("Unable to convert address, see {$target} for details");
                     $rule[$target] = json_encode($rule[$tag]);
                 }
             }
@@ -318,4 +337,55 @@ abstract class Rule
             return false;
         }
     }
+
+    /**
+     * return label
+     * @return string
+     */
+    public function getLabel()
+    {
+        return !empty($this->rule['label']) ? $this->rule['label'] : "";
+    }
+
+    /**
+     * return #ref
+     * @return string
+     */
+    public function getRef()
+    {
+        return !empty($this->rule['#ref']) ? $this->rule['#ref'] : "";
+    }
+
+    /**
+     * return description
+     * @return string
+     */
+    public function getDescr()
+    {
+        return !empty($this->rule['descr']) ? $this->rule['descr'] : "";
+    }
+
+    /**
+     * return interface
+     */
+    public function getInterface()
+    {
+        return !empty($this->rule['interface']) ? $this->rule['interface'] : "";
+    }
+
+    /**
+     * is rule enabled
+     */
+    public function isEnabled()
+    {
+        return empty($this->rule['disabled']);
+    }
+
+    /**
+     * return raw rule
+     */
+    public function getRawRule()
+    {
+        return $this->rule;
+    }
 }
