@@ -1,42 +1,36 @@
 <?php
 
 /*
-    Copyright (C) 2014-2016 Deciso B.V.
-    Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>.
-    All rights reserved.
-
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
-
-    1. Redistributions of source code must retain the above copyright notice,
-       this list of conditions and the following disclaimer.
-
-    2. Redistributions in binary form must reproduce the above copyright
-       notice, this list of conditions and the following disclaimer in the
-       documentation and/or other materials provided with the distribution.
-
-    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
-    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
-*/
+ * Copyright (C) 2014-2016 Deciso B.V.
+ * Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 
 require_once("guiconfig.inc");
 require_once("services.inc");
 require_once("interfaces.inc");
 
-function staticmapcmp($a, $b)
-{
-    return ipcmp($a['ipaddr'], $b['ipaddr']);
-}
-
-
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     // handle identifiers and action
     if (!empty($_GET['if']) && !empty($config['interfaces'][$_GET['if']])) {
@@ -53,8 +47,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig = array();
     $config_copy_fieldnames = array('mac', 'cid', 'hostname', 'filename', 'rootpath', 'descr', 'arp_table_static_entry',
       'defaultleasetime', 'maxleasetime', 'gateway', 'domain', 'domainsearchlist', 'winsserver', 'dnsserver', 'ddnsdomain',
-      'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'ddnsupdate', 'ntpserver', 'tftp', 'ipaddr',
-      'winsserver', 'dnsserver');
+      'ddnsupdate', 'ntpserver', 'tftp', 'bootfilename', 'ipaddr', 'winsserver', 'dnsserver');
     foreach ($config_copy_fieldnames as $fieldname) {
         if (isset($if) && isset($id) && isset($config['dhcpd'][$if]['staticmap'][$id][$fieldname])) {
             $pconfig[$fieldname] = $config['dhcpd'][$if]['staticmap'][$id][$fieldname];
@@ -146,6 +139,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
     }
 
+    $parent_net = find_interface_network(get_real_interface($if));
+
     /* make sure it's not within the dynamic subnet */
     if (!empty($pconfig['ipaddr'])) {
         $dynsubnet_start = ip2ulong($config['dhcpd'][$if]['range']['from']);
@@ -163,31 +158,27 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             }
         }
 
-        $ifcfgip = get_interface_ip($if);
-        $ifcfgsn = get_interface_subnet($if);
-        $ifcfgdescr = convert_friendly_interface_to_friendly_descr($if);
-        $lansubnet_start = ip2ulong(long2ip32(ip2long($ifcfgip) & gen_subnet_mask_long($ifcfgsn)));
-        $lansubnet_end = ip2ulong(long2ip32(ip2long($ifcfgip) | (~gen_subnet_mask_long($ifcfgsn))));
-        if (ip2ulong($pconfig['ipaddr']) < $lansubnet_start || ip2ulong($pconfig['ipaddr']) > $lansubnet_end) {
-            $input_errors[] = sprintf(gettext("The IP address must lie in the %s subnet."),$ifcfgdescr);
+        if (!ip_in_subnet($pconfig['ipaddr'], $parent_net)) {
+            $ifcfgdescr = convert_friendly_interface_to_friendly_descr($if);
+            $input_errors[] = sprintf(gettext('The IP address must lie in the %s subnet.'), $ifcfgdescr);
         }
     }
 
     if (!empty($pconfig['gateway']) && !is_ipaddrv4($pconfig['gateway'])) {
         $input_errors[] = gettext("A valid IP address must be specified for the gateway.");
     }
+
     if ((!empty($pconfig['wins1']) && !is_ipaddrv4($pconfig['wins1'])) ||
       (!empty($pconfig['wins2']) && !is_ipaddrv4($pconfig['wins2']))) {
         $input_errors[] = gettext("A valid IP address must be specified for the primary/secondary WINS servers.");
     }
 
-    $parent_ip = get_interface_ip($pconfig['if']);
-    if (is_ipaddrv4($parent_ip) && $pconfig['gateway']) {
-        $parent_sn = get_interface_subnet($pconfig['if']);
-        if (!ip_in_subnet($pconfig['gateway'], gen_subnet($parent_ip, $parent_sn) . "/" . $parent_sn) && !ip_in_interface_alias_subnet($pconfig['if'], $pconfig['gateway'])) {
+    if (is_subnetv4($parent_net) && !empty($pconfig['gateway'])) {
+        if (!ip_in_subnet($pconfig['gateway'], $parent_net) && !ip_in_interface_alias_subnet($if, $pconfig['gateway'])) {
             $input_errors[] = sprintf(gettext("The gateway address %s does not lie within the chosen interface's subnet."), $_POST['gateway']);
         }
     }
+
     if ((!empty($pconfig['dns1']) && !is_ipaddrv4($pconfig['dns1'])) || (!empty($pconfig['dns2']) && !is_ipaddrv4($pconfig['dns2']))) {
         $input_errors[] = gettext("A valid IP address must be specified for the primary/secondary DNS servers.");
     }
@@ -201,13 +192,6 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     if (!empty($pconfig['ddnsdomain']) && !is_domain($pconfig['ddnsdomain'])) {
         $input_errors[] = gettext("A valid domain name must be specified for the dynamic DNS registration.");
     }
-    if (!empty($pconfig['ddnsdomain']) && !is_ipaddrv4($pconfig['ddnsdomainprimary'])) {
-        $input_errors[] = gettext("A valid primary domain name server IP address must be specified for the dynamic domain name.");
-    }
-    if ((!empty($pconfig['ddnsdomainkey']) && empty($pconfig['ddnsdomainkeyname'])) ||
-      (!empty($pconfig['ddnsdomainkeyname']) && empty($pconfig['ddnsdomainkey']))) {
-        $input_errors[] = gettext("You must specify both a valid domain key and key name.");
-    }
     if (!empty($pconfig['domainsearchlist'])) {
         $domain_array=preg_split("/[ ;]+/", $pconfig['domainsearchlist']);
         foreach ($domain_array as $curdomain) {
@@ -232,17 +216,17 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $mapent = array();
         $config_copy_fieldnames = array('mac', 'cid', 'ipaddr', 'hostname', 'descr', 'filename', 'rootpath',
           'arp_table_static_entry', 'defaultleasetime', 'maxleasetime', 'gateway', 'domain', 'domainsearchlist',
-          'ddnsdomain', 'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'ddnsupdate', 'tftp',
-          'ldap', 'winsserver', 'dnsserver');
+          'ddnsdomain', 'ddnsupdate', 'tftp', 'bootfilename', 'winsserver', 'dnsserver');
 
         foreach ($config_copy_fieldnames as $fieldname) {
             if (!empty($pconfig[$fieldname])) {
                 $mapent[$fieldname] = $pconfig[$fieldname];
             }
         }
+
         // boolean
-        $mapent['arp_table_static_entry'] = !empty($mapent['arp_table_static_entry']);
-        $mapent['ddnsupdate'] = !empty($pconfig['ddnsupdate']) ? true : false;
+        $mapent['arp_table_static_entry'] = !empty($pconfig['arp_table_static_entry']);
+        $mapent['ddnsupdate'] = !empty($pconfig['ddnsupdate']);
 
         // arrays
         $mapent['winsserver'] = array();
@@ -274,8 +258,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         } else {
             $a_maps[] = $mapent;
         }
-        // sort before save
-        usort($config['dhcpd'][$if]['staticmap'], "staticmapcmp");
+
+        usort($config['dhcpd'][$if]['staticmap'], function ($a, $b) {
+            return ipcmp($a['ipaddr'], $b['ipaddr']);
+        });
 
         write_config();
 
@@ -424,7 +410,7 @@ include("head.inc");
                     <input name="dns1" type="text" value="<?=$pconfig['dns1'];?>" /><br/>
                     <input name="dns2" type="text" value="<?=$pconfig['dns2'];?>" />
                     <div class="hidden" data-for="help_for_dns">
-                      <?=gettext("NOTE: leave blank to use the system default DNS servers - this interface's IP if DNS forwarder is enabled, otherwise the servers configured on the General page.");?>
+                      <?= gettext('Leave blank to use the system default DNS servers: This interface IP address if a DNS service is enabled or the configured global DNS servers.') ?>
                     </div>
                   </td>
                 </tr>
@@ -451,7 +437,7 @@ include("head.inc");
                   <td>
                     <input name="domainsearchlist" type="text" id="domainsearchlist" size="20" value="<?=$pconfig['domainsearchlist'];?>" />
                     <div class="hidden" data-for="help_for_domainsearchlist">
-                      <?=gettext("The DHCP server can optionally provide a domain search list. Use the semicolon character as separator ");?>
+                      <?=gettext("The DHCP server can optionally provide a domain search list. Use the semicolon character as separator.");?>
                     </div>
                   </td>
                 </tr>
@@ -479,20 +465,14 @@ include("head.inc");
                   <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Dynamic DNS");?></td>
                   <td>
                     <div id="showddnsbox">
-                      <input type="button" onclick="show_ddns_config()" class="btn btn-xs btn-default" value="<?=gettext("Advanced");?>" /> - <?=gettext("Show Dynamic DNS");?>
+                      <input type="button" onclick="show_ddns_config()" class="btn btn-xs btn-default" value="<?= html_safe(gettext('Advanced')) ?>" /> - <?=gettext("Show Dynamic DNS");?>
                     </div>
                     <div id="showddns" style="display:none">
-                      <input style="vertical-align:middle" type="checkbox" value="yes" name="ddnsupdate" id="ddnsupdate" <?=!empty($pconfig['ddnsupdate']) ? "checked=\"checked\"" : ""; ?> />
+                      <input type="checkbox" value="yes" name="ddnsupdate" id="ddnsupdate" <?=!empty($pconfig['ddnsupdate']) ? "checked=\"checked\"" : ""; ?> />
                       <b><?=gettext("Enable registration of DHCP client names in DNS.");?></b><br />
                       <?=gettext("Note: Leave blank to disable dynamic DNS registration.");?><br />
                       <?=gettext("Enter the dynamic DNS domain which will be used to register client names in the DNS server.");?>
                       <input name="ddnsdomain" type="text" id="ddnsdomain" size="20" value="<?=$pconfig['ddnsdomain'];?>" />
-                      <?=gettext("Enter the primary domain name server IP address for the dynamic domain name.");?><br />
-                      <input name="ddnsdomainprimary" type="text" id="ddnsdomainprimary" size="20" value="<?=$pconfig['ddnsdomainprimary'];?>" />
-                      <?=gettext("Enter the dynamic DNS domain key name which will be used to register client names in the DNS server.");?>
-                      <input name="ddnsdomainkeyname" type="text" id="ddnsdomainkeyname" size="20" value="<?=$pconfig['ddnsdomainkeyname'];?>" />
-                      <?=gettext("Enter the dynamic DNS domain key secret which will be used to register client names in the DNS server.");?>
-                      <input name="ddnsdomainkey" type="text" id="ddnsdomainkey" size="20" value="<?=$pconfig['ddnsdomainkey'];?>" />
                     </div>
                   </td>
                 </tr>
@@ -500,7 +480,7 @@ include("head.inc");
                   <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("NTP servers");?></td>
                   <td>
                     <div id="showntpbox">
-                      <input type="button" onclick="show_ntp_config()" class="btn btn-xs btn-default" value="<?=gettext("Advanced");?>" /> - <?=gettext("Show NTP configuration");?>
+                      <input type="button" onclick="show_ntp_config()" class="btn btn-xs btn-default" value="<?= html_safe(gettext('Advanced')) ?>" /> - <?=gettext("Show NTP configuration");?>
                     </div>
                     <div id="showntp" style="display:none">
                       <input name="ntp1" type="text" id="ntp1" size="20" value="<?=$pconfig['ntp1'];?>" /><br />
@@ -512,19 +492,22 @@ include("head.inc");
                   <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("TFTP server");?></td>
                   <td>
                     <div id="showtftpbox">
-                      <input type="button" onclick="show_tftp_config()" class="btn btn-xs btn-default" value="<?=gettext("Advanced");?>" /> - <?=gettext("Show TFTP configuration");?>
+                      <input type="button" onclick="show_tftp_config()" class="btn btn-xs btn-default" value="<?= html_safe(gettext('Advanced')) ?>" /> - <?=gettext("Show TFTP configuration");?>
                     </div>
                     <div id="showtftp" style="display:none">
-                      <input name="tftp" type="text" id="tftp" size="50" value="<?=$pconfig['tftp'];?>" /><br />
-                      <?=gettext("Leave blank to disable. Enter a full hostname or IP for the TFTP server.");?>
+                      <?=gettext("Set TFTP hostname");?>
+                      <input name="tftp" type="text" size="50" value="<?=$pconfig['tftp'];?>" /><br />
+                      <?=gettext("Set Bootfile");?>
+                      <input name="bootfilename" type="text" value="<?=$pconfig['bootfilename'];?>" /><br />
+                      <?=gettext("Leave blank to disable. Enter a full hostname or IP for the TFTP server and optionally a full path for a bootfile.");?>
                     </div>
                   </td>
                 </tr>
                 <tr>
                   <td></td>
                   <td>
-                    <input name="Submit" type="submit" class="formbtn btn btn-primary" value="<?=gettext("Save");?>" />
-                    <input type="button" class="formbtn btn btn-default" value="<?=gettext("Cancel");?>" onclick="window.location.href='/services_dhcp.php?if=<?= html_safe($if) ?>'" />
+                    <input name="Submit" type="submit" class="formbtn btn btn-primary" value="<?=html_safe(gettext('Save'));?>" />
+                    <input type="button" class="formbtn btn btn-default" value="<?=html_safe(gettext('Cancel'));?>" onclick="window.location.href='/services_dhcp.php?if=<?= html_safe($if) ?>'" />
 <?php
                   if (isset($id)): ?>
                     <input name="id" type="hidden" value="<?=$id;?>" />
